@@ -65,6 +65,21 @@ class DiscreteFactor(Factor):
         new_factor.name = factor.get_name()
         return new_factor
 
+    @staticmethod
+    def from_flat_values(model: GraphModel,
+                         var_idx: List[int],
+                         values_flat: np.ndarray):
+        """Creates factor specified by list of values.
+
+        :param model: GM to which this factor belongs.
+        :param var_idx: Indices of variables.
+        :param values_flat: 1D list of values.
+          Last variable is "least significant".
+        """
+        shape = [model[i].domain.size() for i in var_idx]
+        values = values_flat.reshape(shape)
+        return DiscreteFactor(model, var_idx, values)
+
     def marginal(self, new_var_idx: List[int]) -> DiscreteFactor:
         """Marginalizes factor on subset of variables."""
         assert len(self.var_idx) <= 26
@@ -82,3 +97,22 @@ class DiscreteFactor(Factor):
     def max_marginal(self, new_var_idx: List[int]) -> DiscreteFactor:
         """Marginalizes factor on subset of variables, using MAX-PROD."""
         raise ValueError("Not implemented.")
+
+    def restrict(self, variable_id, fixed_value) -> DiscreteFactor:
+        """Fixes value of one variable.
+
+        Returns new factor which is equivalent to original factor, but with
+        value of one variable fixed.
+        """
+        assert variable_id in self.var_idx
+        if len(self.var_idx) == 1:
+            # TODO: remove this hack - this case should return scalar factor.
+            assert self.var_idx[0] == variable_id
+            new_values = np.zeros_like(self.values)
+            new_values[fixed_value] = self.values[fixed_value]
+            return DiscreteFactor(self.model, [variable_id], new_values)
+        new_var_idx = [i for i in self.var_idx if i != variable_id]
+        idx = tuple(fixed_value if i == variable_id else slice(None) for i in
+                    self.var_idx)
+        new_values = self.values[idx]
+        return DiscreteFactor(self.model, new_var_idx, new_values)
